#![allow(dead_code)]

pub const ACTION_ICONS: &str    = "action-icons";
pub const CATEGORY: &str        = "category";
pub const DESKTOP_ENTRY: &str   = "desktop-entry";
pub const IMAGE_PATH: &str      = "image-path";
pub const RESIDENT: &str        = "resident";
pub const SOUND_FILE: &str      = "sound-file";
pub const SOUND_NAME: &str      = "sound-name";
pub const SUPPRESS_SOUND: &str  = "suppress-sound";
pub const TRANSIENT: &str       = "transient";
pub const X: &str               = "x";
pub const Y: &str               = "y";
pub const URGENCY: &str         = "urgency";


pub const INVALID: &str    = "invalid";